support-info script
===================

This script enables the user to get detailed statistics about every Rudder component, to have a global overview of the current installation.

It works on both Rudder agents and servers/relays, and will run the appropriate tests depending of the environment, unless a specific test is asked for.

How it works
------------

This script can either be totally self-contained (using the bundled Makefile), and/or use the "lib" and "tests" directories to load helper functions and tests.

A helper function only purpose is to return useful values for tests or provide useful features like colorized terminal output.

A test is a file containing tests for a specific component.

A Makefile is available at the project root to create a standalone script version (support-script-standalone), that does not need any of the lib / tests directories to run
(but can also use them to extend builtin tests if necessary, or override them). Just run make and you're all set :)

Test structure
--------------

Please take a look at [tests/skel](tests/skel) for a commented test sample.

The idea is quite simple: a test is basically a shell function containing scenario related to a component, named "<component>_<scenario>[_agent]", that registers itself using the global "TESTS" variable.

The "_agent" suffix is optionnal and tells the test is runnable on both agents and servers, whereas without it it means the test should be run on servers only.

The components classify the scenarios to let the user run only scenarios related to a component.
Ex.: "I want to run only the tests that validate that cfengine is OK"

A test run an arbitrary series of commands to test, and outputs the result using one of the following functions:
* test_success: the test has succeeded
* test_failure: the test has failed
* test_skipped: the test has been skipped, due to being impossible or irrelevant to run (missing tool or wrong conditions)

Those functions will:
* Output a colorized message in the terminal (followed optionnaly by a verbose comment)
* Output a message in the log file
* Increment the successful/failed/skipped test counter that are displayed during the test summary at the end

Usage
-----

```
root@rudder-agent:~# ./support-info  help
This is the rudder support info script.

Usage: ./support-info [help|all|component] [parameters ...]

Components:
* help: this help
* cfengine: cfengine related scenarios
* rudder: rudder related scenarios

Note: If no component is given, run all scenarios in all components.

root@rudder-agent:~# ./support-info rudder
Running all scenarios from component rudder...

Downloading server uuid using http succeeded.......................... OK
(...)
root@rudder-agent:~# ./support-info rudder unsent_inventories_agent
Running the rudder unsent_inventories_agent scenario...

No unsent inventories detected........................................ OK
(...)
```

Sample output
-------------

```
root@rudder-agent:~# ./support-info 
Running all agent scenarios...

cfengine component:

cf-promises on localhost ran successfully............................. OK
CFEngine keys present and usable...................................... OK
Connection to server on port 80 successful............................ OK
Connection to server on port 443 successful........................... OK
Connection to server on port 514 successful........................... OK
Connection to server on port 5309 successful.......................... OK

rudder component:

Downloading server uuid using http succeeded.......................... OK
Downloading server uuid using https failed............................ WARNING
There are unsent inventories.......................................... ERROR

Success: 7
Failure: 1
Skipped: 1
```

This output indicate that the current machine is an agent (no server tests will be run as it has no server role
and had trouble contacting the server using https as well as unsent inventories pending analysis.
It probably has connectivity problems.
